@@ -4,6 +4,7 @@ from __future__ import annotations
 
 from typing import Any
 
+from logfire.propagate import attach_context, get_context
 from logfire.sampling import SamplingOptions
 
 from ._internal.auto_trace import AutoTraceModule
@@ -36,6 +37,7 @@ instrument_openai_agents = DEFAULT_LOGFIRE_INSTANCE.instrument_openai_agents
 instrument_anthropic = DEFAULT_LOGFIRE_INSTANCE.instrument_anthropic
 instrument_google_genai = DEFAULT_LOGFIRE_INSTANCE.instrument_google_genai
 instrument_litellm = DEFAULT_LOGFIRE_INSTANCE.instrument_litellm
+instrument_print = DEFAULT_LOGFIRE_INSTANCE.instrument_print
 instrument_asyncpg = DEFAULT_LOGFIRE_INSTANCE.instrument_asyncpg
 instrument_httpx = DEFAULT_LOGFIRE_INSTANCE.instrument_httpx
 instrument_celery = DEFAULT_LOGFIRE_INSTANCE.instrument_celery
@@ -129,6 +131,7 @@ __all__ = (
     'instrument_anthropic',
     'instrument_google_genai',
     'instrument_litellm',
+    'instrument_print',
     'instrument_asyncpg',
     'instrument_httpx',
     'instrument_celery',
@@ -167,4 +170,6 @@ __all__ = (
     'logfire_info',
     'get_baggage',
     'set_baggage',
+    'get_context',
+    'attach_context',
 )
