@@ -66,9 +66,11 @@ from logfire.sampling._tail_sampling import TailSamplingProcessor
 from logfire.version import VERSION
 
 from ..propagate import NoExtractTraceContextPropagator, WarnOnExtractTraceContextPropagator
+from ..types import ExceptionCallback
 from .client import InvalidProjectName, LogfireClient, ProjectAlreadyExists
 from .config_params import ParamManager, PydanticPluginRecordValues
 from .constants import (
+    LEVEL_NUMBERS,
     RESOURCE_ATTRIBUTES_CODE_ROOT_PATH,
     RESOURCE_ATTRIBUTES_CODE_WORK_DIR,
     RESOURCE_ATTRIBUTES_DEPLOYMENT_ENVIRONMENT_NAME,
@@ -111,6 +113,8 @@ from .utils import (
 )
 
 if TYPE_CHECKING:
+    from typing import TextIO
+
     from opentelemetry._events import EventLoggerProvider
 
     from .main import Logfire
@@ -157,6 +161,8 @@ class ConsoleOptions:
 
     show_project_link: bool = True
     """Whether to print the URL of the Logfire project after initialization."""
+    output: TextIO | None = None
+    """The output stream to write console output to (default: stdout)."""
 
 
 @dataclass
@@ -181,6 +187,11 @@ class AdvancedOptions:
     log_record_processors: Sequence[LogRecordProcessor] = ()
     """Configuration for OpenTelemetry logging. This is experimental and may be removed."""
 
+    exception_callback: ExceptionCallback | None = None
+    """Callback function that is called when an exception is recorded on a span.
+
+    This is experimental and may be modified or removed."""
+
     def generate_base_url(self, token: str) -> str:
         if self.base_url is not None:
             return self.base_url
@@ -268,6 +279,7 @@ def configure(  # noqa: D417
     scrubbing: ScrubbingOptions | Literal[False] | None = None,
     inspect_arguments: bool | None = None,
     sampling: SamplingOptions | None = None,
+    min_level: int | LevelName | None = None,
     add_baggage_to_attributes: bool = True,
     code_source: CodeSource | None = None,
     distributed_tracing: bool | None = None,
@@ -320,6 +332,16 @@ def configure(  # noqa: D417
 
             Defaults to `True` if and only if the Python version is at least 3.11.
 
+            Also enables magic argument inspection in [`logfire.instrument_print()`][logfire.Logfire.instrument_print].
+
+        min_level:
+            Minimum log level for logs and spans to be created. By default, all logs and spans are created.
+            For example, set to 'info' to only create logs with level 'info' or higher, thus filtering out debug logs.
+            For spans, this only applies when `_level` is explicitly specified in `logfire.span`.
+            Changing the level of a span _after_ it is created will be ignored by this.
+            If a span is not created, this has no effect on the current active span, or on logs/spans created inside the
+            filtered `logfire.span` context manager.
+            If set to `None`, uses the `LOGFIRE_MIN_LEVEL` environment variable; if that is not set, there is no minimum level.
         sampling: Sampling options. See the [sampling guide](https://logfire.pydantic.dev/docs/guides/advanced/sampling/).
         add_baggage_to_attributes: Set to `False` to prevent OpenTelemetry Baggage from being added to spans as attributes.
             See the [Baggage documentation](https://logfire.pydantic.dev/docs/reference/advanced/baggage/) for more details.
@@ -456,6 +478,7 @@ def configure(  # noqa: D417
         additional_span_processors=additional_span_processors,
         scrubbing=scrubbing,
         inspect_arguments=inspect_arguments,
+        min_level=min_level,
         sampling=sampling,
         add_baggage_to_attributes=add_baggage_to_attributes,
         code_source=code_source,
@@ -514,6 +537,9 @@ class _LogfireConfigData:
     sampling: SamplingOptions
     """Sampling options."""
 
+    min_level: int
+    """Minimum log level for logs and spans to be created."""
+
     add_baggage_to_attributes: bool
     """Whether to add OpenTelemetry Baggage to span attributes."""
 
@@ -544,6 +570,7 @@ class _LogfireConfigData:
         scrubbing: ScrubbingOptions | Literal[False] | None,
         inspect_arguments: bool | None,
         sampling: SamplingOptions | None,
+        min_level: int | LevelName | None,
         add_baggage_to_attributes: bool,
         code_source: CodeSource | None,
         distributed_tracing: bool | None,
@@ -561,6 +588,12 @@ class _LogfireConfigData:
         self.inspect_arguments = param_manager.load_param('inspect_arguments', inspect_arguments)
         self.distributed_tracing = param_manager.load_param('distributed_tracing', distributed_tracing)
         self.ignore_no_config = param_manager.load_param('ignore_no_config')
+        min_level = param_manager.load_param('min_level', min_level)
+        if min_level is None:
+            min_level = 0
+        elif isinstance(min_level, str):
+            min_level = LEVEL_NUMBERS[min_level]
+        self.min_level = min_level
         self.add_baggage_to_attributes = add_baggage_to_attributes
 
         # We save `scrubbing` just so that it can be serialized and deserialized.
@@ -647,6 +680,7 @@ class LogfireConfig(_LogfireConfigData):
         scrubbing: ScrubbingOptions | Literal[False] | None = None,
         inspect_arguments: bool | None = None,
         sampling: SamplingOptions | None = None,
+        min_level: int | LevelName | None = None,
         add_baggage_to_attributes: bool = True,
         code_source: CodeSource | None = None,
         distributed_tracing: bool | None = None,
@@ -674,6 +708,7 @@ class LogfireConfig(_LogfireConfigData):
             scrubbing=scrubbing,
             inspect_arguments=inspect_arguments,
             sampling=sampling,
+            min_level=min_level,
             add_baggage_to_attributes=add_baggage_to_attributes,
             code_source=code_source,
             distributed_tracing=distributed_tracing,
@@ -712,6 +747,7 @@ class LogfireConfig(_LogfireConfigData):
         scrubbing: ScrubbingOptions | Literal[False] | None,
         inspect_arguments: bool | None,
         sampling: SamplingOptions | None,
+        min_level: int | LevelName | None,
         add_baggage_to_attributes: bool,
         code_source: CodeSource | None,
         distributed_tracing: bool | None,
@@ -733,6 +769,7 @@ class LogfireConfig(_LogfireConfigData):
                 scrubbing,
                 inspect_arguments,
                 sampling,
+                min_level,
                 add_baggage_to_attributes,
                 code_source,
                 distributed_tracing,
@@ -861,6 +898,7 @@ class LogfireConfig(_LogfireConfigData):
                     include_tags=self.console.include_tags,
                     verbose=self.console.verbose,
                     min_log_level=self.console.min_log_level,
+                    output=self.console.output,
                 )
                 add_span_processor(SimpleSpanProcessor(console_span_exporter))
                 log_record_processors.append(SimpleLogRecordProcessor(ConsoleLogExporter(console_span_exporter)))
@@ -870,30 +908,52 @@ class LogfireConfig(_LogfireConfigData):
                 metric_readers = list(self.metrics.additional_readers)
 
             if self.send_to_logfire:
-                credentials: LogfireCredentials | None = None
                 show_project_link: bool = self.console and self.console.show_project_link or False
 
-                # try loading credentials (and thus token) from file if a token is not already available
-                # this takes the lowest priority, behind the token passed to `configure` and the environment variable
-                if not self.token:
+                # Try loading credentials from a file.
+                # If that works, we can use it to immediately print the project link.
+                try:
                     credentials = LogfireCredentials.load_creds_file(self.data_dir)
+                except Exception:
+                    # If we have a token configured by other means, e.g. the env, no need to worry about the creds file.
+                    if not self.token:
+                        raise
+                    credentials = None
 
-                    # if we still don't have a token, try initializing a new project and writing a new creds file
+                if not self.token and self.send_to_logfire is True and credentials is None:
+                    # If we don't have a token or credentials from a file,
+                    # try initializing a new project and writing a new creds file.
                     # note, we only do this if `send_to_logfire` is explicitly `True`, not 'if-token-present'
-                    if self.send_to_logfire is True and credentials is None:
-                        client = LogfireClient.from_url(self.advanced.base_url)
-                        credentials = LogfireCredentials.initialize_project(client=client)
-                        credentials.write_creds_file(self.data_dir)
+                    client = LogfireClient.from_url(self.advanced.base_url)
+                    credentials = LogfireCredentials.initialize_project(client=client)
+                    credentials.write_creds_file(self.data_dir)
 
-                    if credentials is not None:
-                        self.token = credentials.token
-                        self.advanced.base_url = self.advanced.base_url or credentials.logfire_api_url
+                if credentials is not None:
+                    # Get token and base_url from credentials if not already set.
+                    # This means that e.g. a token in an env var takes priority over a token in a creds file.
+                    self.token = self.token or credentials.token
+                    self.advanced.base_url = self.advanced.base_url or credentials.logfire_api_url
 
                 if self.token:
+                    if credentials and self.token == credentials.token and show_project_link:
+                        # The creds file contains the project link, so we can display it immediately.
+                        # We do this if the token comes from the creds file or if it was explicitly configured
+                        # and happens to match the creds file anyway.
+                        credentials.print_token_summary()
+                        # Don't print it again in check_token below.
+                        show_project_link = False
+
+                    # Regardless of where the token comes from, check that it's valid.
+                    # Even if it comes from a creds file, it could be revoked or expired.
+                    # If it's valid and we haven't already printed a project link, print it here.
+                    # This may happen some time later in a background thread which can be annoying,
+                    # hence we try to print it eagerly above.
+                    # But we only have the link if we have a creds file, otherwise we only know the token at this point.
 
                     def check_token():
                         assert self.token is not None
-                        validated_credentials = self._initialize_credentials_from_token(self.token)
+                        with suppress_instrumentation():
+                            validated_credentials = self._initialize_credentials_from_token(self.token)
                         if show_project_link and validated_credentials is not None:
                             validated_credentials.print_token_summary()
 
@@ -1053,6 +1113,7 @@ class LogfireConfig(_LogfireConfigData):
                 self._logger_provider.shutdown()
 
             self._logger_provider.set_provider(logger_provider)
+            self._logger_provider.set_min_level(self.min_level)
 
             if self is GLOBAL_CONFIG and not self._has_set_providers:
                 self._has_set_providers = True
@@ -1168,7 +1229,9 @@ class LogfireConfig(_LogfireConfigData):
         return self._event_logger_provider
 
     def warn_if_not_initialized(self, message: str):
-        if not self._initialized and not self.ignore_no_config:
+        ignore_no_config_env = os.getenv('LOGFIRE_IGNORE_NO_CONFIG', '')
+        ignore_no_config = ignore_no_config_env.lower() in ('1', 'true', 't') or self.ignore_no_config
+        if not self._initialized and not ignore_no_config:
             warn_at_user_stacklevel(
                 f'{message} until `logfire.configure()` has been called. '
                 f'Set the environment variable LOGFIRE_IGNORE_NO_CONFIG=1 or add ignore_no_config=true in pyproject.toml to suppress this warning.',
