@@ -12,6 +12,7 @@ from typing import TYPE_CHECKING, Any, Callable, Literal, cast
 import httpx
 from opentelemetry.trace import NonRecordingSpan, Span, use_span
 
+from logfire._internal.config import GLOBAL_CONFIG
 from logfire._internal.stack_info import warn_at_user_stacklevel
 
 try:
@@ -45,7 +46,7 @@ if TYPE_CHECKING:
 def instrument_httpx(
     logfire_instance: Logfire,
     client: httpx.Client | httpx.AsyncClient | None,
-    capture_all: bool,
+    capture_all: bool | None,
     capture_headers: bool,
     capture_request_body: bool,
     capture_response_body: bool,
@@ -76,6 +77,8 @@ def instrument_httpx(
             'The `capture_response_headers` parameter is deprecated. Use `capture_headers` instead.', DeprecationWarning
         )
 
+    capture_all = cast(bool, GLOBAL_CONFIG.param_manager.load_param('httpx_capture_all', capture_all))
+
     should_capture_request_headers = capture_request_headers or capture_headers or capture_all
     should_capture_response_headers = capture_response_headers or capture_headers or capture_all
     should_capture_request_body = capture_request_body or capture_all
