@@ -2,8 +2,9 @@
 
 from __future__ import annotations
 
+from collections.abc import Mapping
 from logging import NOTSET, Handler as LoggingHandler, LogRecord, StreamHandler
-from typing import TYPE_CHECKING, Any, ClassVar, Mapping, cast
+from typing import TYPE_CHECKING, Any, ClassVar, cast
 
 import logfire
 
@@ -51,7 +52,13 @@ if TYPE_CHECKING:
 
 
 class LogfireLoggingHandler(LoggingHandler):
-    """A [logging](../../integrations/logging.md) handler that sends logs to **Logfire**."""
+    """A [logging](../../integrations/logging.md) handler that sends logs to **Logfire**.
+
+    Args:
+        level: The threshold level for this handler. Logging messages which are less severe than *level* will be ignored.
+        fallback: A fallback handler to use when [instrumentation is suppressed](../../how-to-guides/suppress.md#suppress-instrumentation).
+        logfire_instance: The Logfire instance to use when emitting logs. Defaults to the default global instance.
+    """
 
     custom_scope_suffix: ClassVar[str] = 'stdlib.logging'
 
