@@ -2,9 +2,11 @@ import json
 import logging
 import warnings
 from typing import Any
+from unittest import mock
 
 import pydantic
 import pytest
+from dirty_equals import IsPartialDict
 from inline_snapshot import snapshot
 
 import logfire
@@ -13,6 +15,17 @@ from logfire._internal.utils import get_version
 from logfire.testing import TestExporter
 
 
+def test_missing_opentelemetry_dependency() -> None:
+    with mock.patch.dict('sys.modules', {'openinference.instrumentation.litellm': None}):
+        with pytest.raises(RuntimeError) as exc_info:
+            logfire.instrument_litellm()
+        assert str(exc_info.value) == snapshot("""\
+The `logfire.instrument_litellm()` method requires the `openinference-instrumentation-litellm` package.
+You can install this with:
+    pip install 'logfire[litellm]'\
+""")
+
+
 @pytest.mark.vcr()
 @pytest.mark.skipif(get_version(pydantic.__version__) < get_version('2.5.0'), reason='Requires newer pydantic version')
 def test_litellm_instrumentation(exporter: TestExporter) -> None:
@@ -91,6 +104,7 @@ def test_litellm_instrumentation(exporter: TestExporter) -> None:
                     'logfire.span_type': 'span',
                     'logfire.msg': 'completion',
                     'llm.model_name': 'gpt-4o-mini',
+                    'llm.provider': 'openai',
                     'llm.input_messages.0.message.role': 'user',
                     'llm.input_messages.0.message.content': "What's the weather like in San Francisco?",
                     'input.value': {
@@ -171,18 +185,8 @@ def test_litellm_instrumentation(exporter: TestExporter) -> None:
                             'completion_tokens': 18,
                             'prompt_tokens': 80,
                             'total_tokens': 98,
-                            'completion_tokens_details': {
-                                'accepted_prediction_tokens': 0,
-                                'audio_tokens': 0,
-                                'reasoning_tokens': 0,
-                                'rejected_prediction_tokens': 0,
-                            },
-                            'prompt_tokens_details': {
-                                'audio_tokens': 0,
-                                'cached_tokens': 0,
-                                'text_tokens': None,
-                                'image_tokens': None,
-                            },
+                            'completion_tokens_details': IsPartialDict(),
+                            'prompt_tokens_details': IsPartialDict(),
                         },
                         'service_tier': 'default',
                     },
@@ -243,6 +247,7 @@ def test_litellm_instrumentation(exporter: TestExporter) -> None:
                     'logfire.span_type': 'span',
                     'logfire.msg': 'completion',
                     'llm.model_name': 'gpt-4o-mini',
+                    'llm.provider': 'openai',
                     'llm.input_messages.0.message.role': 'user',
                     'llm.input_messages.0.message.content': "What's the weather like in San Francisco?",
                     'llm.input_messages.1.message.role': 'assistant',
@@ -394,4 +399,4 @@ def test_guess_system():
     assert guess_system('gemini-1.5-pro') == 'google'
     assert guess_system('claude-sonnet') == 'anthropic'
     assert guess_system('other-anthropic') == 'anthropic'
-    assert guess_system('unknown-model') == 'litellm'
+    assert guess_system('unknown-model', 'litellm') == 'litellm'
