@@ -35,7 +35,7 @@ from agents.tracing.span_data import MCPListToolsSpanData, ResponseSpanData
 from agents.tracing.spans import NoOpSpan
 from agents.tracing.traces import NoOpTrace
 from agents.voice import AudioInput, SingleAgentVoiceWorkflow, VoicePipeline
-from dirty_equals import IsInt, IsStr
+from dirty_equals import IsPartialDict, IsStr
 from inline_snapshot import snapshot
 from openai import AsyncOpenAI
 
@@ -44,6 +44,7 @@ from logfire._internal.exporters.test import TestExporter
 from logfire._internal.integrations.openai_agents import LogfireSpanWrapper, LogfireTraceWrapper
 
 os.environ.setdefault('OPENAI_API_KEY', 'foo')
+os.environ['OPENAI_DEFAULT_MODEL'] = 'gpt-4o'
 
 
 def test_openai_agent_tracing(exporter: TestExporter):
@@ -407,7 +408,8 @@ def without_code_attrs(spans: list[dict[str, Any]]) -> list[dict[str, Any]]:
 def simplify_spans(spans: list[dict[str, Any]]) -> list[dict[str, Any]]:
     result: list[dict[str, Any]] = []
     for span in spans:
-        assert isinstance(span['attributes'].pop('logfire.json_schema'), dict)
+        span['attributes'].pop('logfire.json_schema', None)
+        span['attributes'].pop('logfire.metrics', None)
         if span['name'] == 'Responses API with {gen_ai.request.model!r}':
             # These are huge and constantly change when OpenAI updates
             assert isinstance(span['attributes'].pop('response'), dict)
@@ -873,26 +875,7 @@ async def test_chat_completions(exporter: TestExporter):
                             'annotations': [],
                         }
                     ],
-                    'model_config': {
-                        'temperature': None,
-                        'top_p': None,
-                        'frequency_penalty': None,
-                        'presence_penalty': None,
-                        'tool_choice': None,
-                        'parallel_tool_calls': None,
-                        'truncation': None,
-                        'max_tokens': None,
-                        'reasoning': None,
-                        'metadata': None,
-                        'store': None,
-                        'include_usage': None,
-                        'response_include': None,
-                        'extra_query': None,
-                        'extra_body': None,
-                        'extra_headers': None,
-                        'extra_args': None,
-                        'base_url': 'https://api.openai.com/v1/',
-                    },
+                    'model_config': IsPartialDict(),
                     'usage': {'input_tokens': 11, 'output_tokens': 8},
                     'gen_ai.system': 'openai',
                     'gen_ai.request.model': 'gpt-4o',
@@ -1558,7 +1541,7 @@ async def test_function_tool_exception(exporter: TestExporter):
                 'context': {'trace_id': 1, 'span_id': 7, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
                 'start_time': 5000000000,
-                'end_time': 6000000000,
+                'end_time': 7000000000,
                 'attributes': {
                     'logfire.msg_template': 'Function: {name}',
                     'logfire.span_type': 'span',
@@ -1577,7 +1560,7 @@ async def test_function_tool_exception(exporter: TestExporter):
                 'events': [
                     {
                         'name': 'exception',
-                        'timestamp': IsInt(),
+                        'timestamp': 6000000000,
                         'attributes': {
                             'exception.type': 'RuntimeError',
                             'exception.message': "Ouch, don't do that again!",
@@ -1591,8 +1574,8 @@ async def test_function_tool_exception(exporter: TestExporter):
                 'name': 'Responses API with {gen_ai.request.model!r}',
                 'context': {'trace_id': 1, 'span_id': 9, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
-                'start_time': 7000000000,
-                'end_time': 8000000000,
+                'start_time': 8000000000,
+                'end_time': 9000000000,
                 'attributes': {
                     'code.filepath': 'test_openai_agents.py',
                     'code.function': 'test_function_tool_exception',
@@ -1656,7 +1639,7 @@ async def test_function_tool_exception(exporter: TestExporter):
                 'context': {'trace_id': 1, 'span_id': 3, 'is_remote': False},
                 'parent': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
                 'start_time': 2000000000,
-                'end_time': 9000000000,
+                'end_time': 10000000000,
                 'attributes': {
                     'code.filepath': 'test_openai_agents.py',
                     'code.function': 'test_function_tool_exception',
@@ -1676,7 +1659,7 @@ async def test_function_tool_exception(exporter: TestExporter):
                 'context': {'trace_id': 1, 'span_id': 1, 'is_remote': False},
                 'parent': None,
                 'start_time': 1000000000,
-                'end_time': 10000000000,
+                'end_time': 11000000000,
                 'attributes': {
                     'code.filepath': 'test_openai_agents.py',
                     'code.function': 'test_function_tool_exception',
