@@ -7,6 +7,7 @@ import sys
 import threading
 from collections.abc import Iterable, Sequence
 from contextlib import ExitStack
+from io import StringIO
 from pathlib import Path
 from time import sleep, time
 from typing import Any
@@ -583,11 +584,7 @@ def get_batch_span_exporter(processor: SpanProcessor) -> SpanExporter:
 
 def get_batch_log_exporter(processor: LogRecordProcessor) -> LogExporter:
     assert isinstance(processor, BatchLogRecordProcessor)
-    try:
-        exporter = processor._batch_processor._exporter  # type: ignore
-    except AttributeError:
-        exporter = processor._exporter  # type: ignore
-    return exporter  # type: ignore
+    return processor._batch_processor._exporter  # type: ignore
 
 
 def test_configure_export_delay() -> None:
@@ -885,6 +882,19 @@ def test_config_serializable_console_false():
     assert GLOBAL_CONFIG.console is False
 
 
+def test_config_console_output_set():
+    output = StringIO()
+    logfire.configure(send_to_logfire=False, console=logfire.ConsoleOptions(output=output))
+    assert isinstance(GLOBAL_CONFIG.console, logfire.ConsoleOptions)
+    assert GLOBAL_CONFIG.console.output is output
+
+    deserialize_config(serialize_config())
+    assert isinstance(GLOBAL_CONFIG.console, logfire.ConsoleOptions)
+    assert GLOBAL_CONFIG.console.output is output
+    logfire.info('test')
+    assert 'test' in output.getvalue()
+
+
 def test_sanitize_project_name():
     assert sanitize_project_name('foo') == 'foo'
     assert sanitize_project_name('FOO') == 'foo'
@@ -909,9 +919,10 @@ def test_initialize_project_use_existing_project_no_projects(tmp_dir_cwd: Path,
 
         request_mocker = requests_mock.Mocker()
         stack.enter_context(request_mocker)
-        request_mocker.get('https://logfire-api.pydantic.dev/v1/projects/', json=[])
+        request_mocker.get('https://logfire-api.pydantic.dev/v1/writable-projects/', json=[])
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/organizations/', json=[{'organization_name': 'fake_org'}]
+            'https://logfire-api.pydantic.dev/v1/organizations/available-for-projects/',
+            json=[{'organization_name': 'fake_org'}],
         )
         request_mocker.get(
             'https://logfire-api.pydantic.dev/v1/info',
@@ -949,7 +960,7 @@ def test_initialize_project_use_existing_project(tmp_dir_cwd: Path, tmp_path: Pa
         request_mocker = requests_mock.Mocker()
         stack.enter_context(request_mocker)
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/projects/',
+            'https://logfire-api.pydantic.dev/v1/writable-projects/',
             json=[{'organization_name': 'fake_org', 'project_name': 'fake_project'}],
         )
         request_mocker.get(
@@ -1007,14 +1018,15 @@ def test_initialize_project_not_using_existing_project(
         request_mocker = requests_mock.Mocker()
         stack.enter_context(request_mocker)
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/organizations/', json=[{'organization_name': 'fake_org'}]
+            'https://logfire-api.pydantic.dev/v1/organizations/available-for-projects/',
+            json=[{'organization_name': 'fake_org'}],
         )
         request_mocker.get(
             'https://logfire-api.pydantic.dev/v1/info',
             json={'project_name': 'myproject', 'project_url': 'fake_project_url'},
         )
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/projects/',
+            'https://logfire-api.pydantic.dev/v1/writable-projects/',
             json=[{'organization_name': 'fake_org', 'project_name': 'fake_project'}],
         )
         create_project_response = {
@@ -1065,10 +1077,11 @@ def test_initialize_project_not_confirming_organization(tmp_path: Path) -> None:
         request_mocker = requests_mock.Mocker()
         stack.enter_context(request_mocker)
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/organizations/', json=[{'organization_name': 'fake_org'}]
+            'https://logfire-api.pydantic.dev/v1/organizations/available-for-projects/',
+            json=[{'organization_name': 'fake_org'}],
         )
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/projects/',
+            'https://logfire-api.pydantic.dev/v1/writable-projects/',
             json=[{'organization_name': 'fake_org', 'project_name': 'fake_project'}],
         )
 
@@ -1106,9 +1119,10 @@ def test_initialize_project_create_project(tmp_dir_cwd: Path, tmp_path: Path, ca
 
         request_mocker = requests_mock.Mocker()
         stack.enter_context(request_mocker)
-        request_mocker.get('https://logfire-api.pydantic.dev/v1/projects/', json=[])
+        request_mocker.get('https://logfire-api.pydantic.dev/v1/writable-projects/', json=[])
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/organizations/', json=[{'organization_name': 'fake_org'}]
+            'https://logfire-api.pydantic.dev/v1/organizations/available-for-projects/',
+            json=[{'organization_name': 'fake_org'}],
         )
         request_mocker.get(
             'https://logfire-api.pydantic.dev/v1/info',
@@ -1157,12 +1171,15 @@ def test_initialize_project_create_project(tmp_dir_cwd: Path, tmp_path: Path, ca
         )
 
         logfire.configure(send_to_logfire=True)
+        assert capsys.readouterr().err == 'Logfire project URL: fake_project_url\n'
 
-        for request in request_mocker.request_history[:-1]:
+        for request in request_mocker.request_history[:5]:
             assert request.headers['Authorization'] == 'fake_user_token'
+        assert len(request_mocker.request_history) in (5, 6)
 
         # we check that fake_token is valid now when we configure the project
         wait_for_check_token_thread()
+        assert len(request_mocker.request_history) == 6
         assert request_mocker.request_history[-1].headers['Authorization'] == 'fake_token'
 
         assert request_mocker.request_history[2].json() == create_existing_project_request_json
@@ -1199,7 +1216,6 @@ def test_initialize_project_create_project(tmp_dir_cwd: Path, tmp_path: Path, ca
                 'Project initialized successfully. You will be able to view it at: fake_project_url\nPress Enter to continue',
             ),
         ]
-        assert capsys.readouterr().err == 'Logfire project URL: fake_project_url\n'
 
         assert json.loads((tmp_dir_cwd / '.logfire/logfire_credentials.json').read_text()) == {
             **create_project_response['json'],
@@ -1222,9 +1238,9 @@ def test_initialize_project_create_project_default_organization(tmp_dir_cwd: Pat
         request_mocker = requests_mock.Mocker()
         stack.enter_context(request_mocker)
         # request_mocker.get('https://logfire-api.pydantic.dev/v1/info', json={'project_name': 'myproject'})
-        request_mocker.get('https://logfire-api.pydantic.dev/v1/projects/', json=[])
+        request_mocker.get('https://logfire-api.pydantic.dev/v1/writable-projects/', json=[])
         request_mocker.get(
-            'https://logfire-api.pydantic.dev/v1/organizations/',
+            'https://logfire-api.pydantic.dev/v1/organizations/available-for-projects/',
             json=[{'organization_name': 'fake_org'}, {'organization_name': 'fake_org1'}],
         )
         request_mocker.get(
@@ -1395,9 +1411,9 @@ def test_send_to_logfire_if_token_present_in_logfire_dir(tmp_path: Path, capsys:
             json={'project_name': 'myproject', 'project_url': 'https://logfire-us.pydantic.dev'},
         )
         configure(send_to_logfire='if-token-present', data_dir=tmp_path)
+        assert capsys.readouterr().err == 'Logfire project URL: https://logfire-us.pydantic.dev\n'
         wait_for_check_token_thread()
         assert len(request_mocker.request_history) == 1
-        assert capsys.readouterr().err == 'Logfire project URL: https://logfire-us.pydantic.dev\n'
 
 
 def test_configure_unknown_token_region(capsys: pytest.CaptureFixture[str]) -> None:
@@ -1418,7 +1434,51 @@ def test_load_creds_file_invalid_json_content(tmp_path: Path):
     creds_file.write_text('invalid-data')
 
     with pytest.raises(LogfireConfigError, match='Invalid credentials file:'):
-        LogfireCredentials.load_creds_file(creds_dir=tmp_path)
+        logfire.configure(data_dir=tmp_path, send_to_logfire=True)
+
+
+def test_load_creds_file_invalid_json_content_with_token_present(tmp_path: Path, capsys: pytest.CaptureFixture[str]):
+    creds_file = tmp_path / 'logfire_credentials.json'
+    creds_file.write_text('invalid-data')
+
+    with patch.dict(os.environ, {'LOGFIRE_TOKEN': 'fake_token'}), requests_mock.Mocker() as request_mocker:
+        request_mocker.get(
+            'https://logfire-us.pydantic.dev/v1/info',
+            json={'project_name': 'myproject', 'project_url': 'fake_project_url'},
+        )
+        logfire.configure(data_dir=tmp_path, send_to_logfire=True)
+        wait_for_check_token_thread()
+        assert len(request_mocker.request_history) == 1
+        assert request_mocker.request_history[0].headers['Authorization'] == 'fake_token'
+        assert capsys.readouterr().err == 'Logfire project URL: fake_project_url\n'
+
+
+def test_load_creds_file_with_token_different_from_env(tmp_path: Path, capsys: pytest.CaptureFixture[str]):
+    creds_file = tmp_path / 'logfire_credentials.json'
+    creds_file.write_text(
+        """
+        {
+            "token": "foobar",
+            "project_name": "myproject",
+            "project_url": "https://logfire-us.pydantic.dev",
+            "logfire_api_url": "https://logfire-us.pydantic.dev"
+        }
+        """
+    )
+
+    with patch.dict(os.environ, {'LOGFIRE_TOKEN': 'fake_token'}), requests_mock.Mocker() as request_mocker:
+        request_mocker.get(
+            'https://logfire-us.pydantic.dev/v1/info',
+            json={'project_name': 'myproject', 'project_url': 'fake_project_url'},
+        )
+        logfire.configure(data_dir=tmp_path, send_to_logfire=True)
+        # not 'foobar' from the creds file. The token in the env var takes precedence.
+        assert logfire.DEFAULT_LOGFIRE_INSTANCE.config.token == 'fake_token'
+
+        wait_for_check_token_thread()
+        assert len(request_mocker.request_history) == 1
+        assert request_mocker.request_history[0].headers['Authorization'] == 'fake_token'
+        assert capsys.readouterr().err == 'Logfire project URL: fake_project_url\n'
 
 
 def test_load_creds_file_legacy_key(tmp_path: Path):
@@ -1443,7 +1503,7 @@ def test_load_creds_file_invalid_key(tmp_path: Path):
     creds_file.write_text('{"test": "test"}')
 
     with pytest.raises(LogfireConfigError, match='Invalid credentials file:'):
-        LogfireCredentials.load_creds_file(creds_dir=tmp_path)
+        logfire.configure(data_dir=tmp_path, send_to_logfire=True)
 
 
 def test_initialize_credentials_from_token_unreachable():
