@@ -4,8 +4,11 @@ from __future__ import annotations
 import decimal
 import enum
 import io
+import json
+import re
 import sys
 from datetime import datetime
+from enum import Enum
 from typing import Any
 from unittest import mock
 
@@ -13,11 +16,10 @@ import pytest
 from dirty_equals import IsStr
 from inline_snapshot import snapshot
 from opentelemetry import trace
-from opentelemetry._events import Event, get_event_logger
 from opentelemetry._logs import SeverityNumber, get_logger
 from opentelemetry.sdk._logs import LogRecord
 from opentelemetry.sdk.trace import ReadableSpan
-from opentelemetry.version import __version__ as otel_version
+from opentelemetry.trace import get_tracer
 
 import logfire
 from logfire import ConsoleOptions
@@ -27,10 +29,18 @@ from logfire._internal.exporters.console import (
     ShowParentsConsoleSpanExporter,
     SimpleConsoleSpanExporter,
 )
-from logfire._internal.utils import get_version
 from logfire.testing import TestExporter
 from tests.utils import ReadableSpanModel, SpanContextModel, exported_spans_as_models
 
+if sys.version_info >= (3, 11):  # pragma: no branch
+    from enum import IntEnum, StrEnum
+else:  # pragma: no cover
+
+    class StrEnum(str, Enum): ...
+
+    class IntEnum(int, Enum): ...
+
+
 tracer = trace.get_tracer('test')
 
 NANOSECONDS_PER_SECOND = int(1e9)
@@ -728,6 +738,7 @@ def test_exception(exporter: TestExporter) -> None:
                     'code.lineno': 123,
                     'a': 'test',
                     'logfire.json_schema': '{"type":"object","properties":{"a":{}}}',
+                    'logfire.exception.fingerprint': '0000000000000000000000000000000000000000000000000000000000000000',
                 },
                 events=[
                     {
@@ -890,51 +901,28 @@ def test_console_otel_logs(capsys: pytest.CaptureFixture[str]):
         console=ConsoleOptions(colors='never', include_timestamps=False, include_tags=False),
     )
     logger = get_logger('logs')
-    event_logger = get_event_logger('events')
 
     with logfire.span('span'):
-        if get_version(otel_version) >= get_version('1.35.0'):
-            logger.emit(
-                LogRecord(
-                    event_name='my_event',
-                    severity_number=SeverityNumber.ERROR,
-                    body='body',
-                    attributes={'key': 'value'},
-                )
+        logger.emit(
+            LogRecord(
+                event_name='my_event',
+                severity_number=SeverityNumber.ERROR,
+                body='body',
+                attributes={'key': 'value'},
             )
-            logger.emit(
-                LogRecord(
-                    event_name='my_event',
-                    attributes={ATTRIBUTES_MESSAGE_KEY: 'msg'},
-                )
+        )
+        logger.emit(
+            LogRecord(
+                event_name='my_event',
+                attributes={ATTRIBUTES_MESSAGE_KEY: 'msg'},
             )
-            logger.emit(
-                LogRecord(
-                    severity_number=SeverityNumber.INFO,
-                    attributes={'key': 'value'},
-                )
-            )
-        else:
-            event_logger.emit(
-                Event(
-                    name='my_event',
-                    severity_number=SeverityNumber.ERROR,
-                    body='body',
-                    attributes={'key': 'value'},
-                )
-            )
-            event_logger.emit(
-                Event(
-                    name='my_event',
-                    attributes={ATTRIBUTES_MESSAGE_KEY: 'msg'},
-                )
-            )
-            logger.emit(
-                LogRecord(
-                    severity_number=SeverityNumber.INFO,
-                    attributes={'key': 'value'},
-                )
+        )
+        logger.emit(
+            LogRecord(
+                severity_number=SeverityNumber.INFO,
+                attributes={'key': 'value'},
             )
+        )
 
     assert capsys.readouterr().out.splitlines() == snapshot(
         [
@@ -993,6 +981,7 @@ def test_other_json_schema_types(capsys: pytest.CaptureFixture[str]) -> None:
         e=MyEnum.ABC,
         se=MyStrEnum.STR,
         ie=MyIntEnum.INT,
+        rp=re.compile(r'^[\w\.-]+@[\w\.-]+\.\w+$'),
     )
 
     assert capsys.readouterr().out.splitlines() == snapshot(
@@ -1005,5 +994,120 @@ def test_other_json_schema_types(capsys: pytest.CaptureFixture[str]) -> None:
             "│ e=MyEnum('abc')",
             "│ se=MyStrEnum('str_val')",
             '│ ie=MyIntEnum(1)',
+            '│ rp=^[\\w\\.-]+@[\\w\\.-]+\\.\\w+$',
+        ]
+    )
+
+
+def test_console_exporter_list_data_with_object_schema_mismatch(capsys: pytest.CaptureFixture[str]) -> None:
+    logfire.configure(
+        send_to_logfire=False,
+        console=ConsoleOptions(verbose=True, colors='never', include_timestamps=False),
+    )
+
+    get_tracer(__name__).start_span(
+        'test_span',
+        attributes={
+            'foo': json.dumps(['item1', 'item2', 'item3']),
+            'bar': json.dumps({'name': 'Alice', 'age': 30}),
+            'logfire.json_schema': json.dumps(
+                {
+                    'type': 'object',
+                    'properties': {
+                        # These are wrong
+                        'foo': {'type': 'object'},
+                        'bar': {'type': 'array'},
+                    },
+                }
+            ),
+        },
+    ).end()
+
+    assert capsys.readouterr().out.splitlines() == snapshot(
+        [
+            'test_span',
+            "│ foo=['item1', 'item2', 'item3']",
+            "│ bar={'name': 'Alice', 'age': 30}",
+        ]
+    )
+
+
+def test_console_exporter_log_pydantic_root_model(capsys: pytest.CaptureFixture[str]) -> None:
+    from pydantic import BaseModel, RootModel
+
+    logfire.configure(
+        send_to_logfire=False,
+        console=ConsoleOptions(verbose=True, colors='never', include_timestamps=False),
+    )
+
+    class Model(BaseModel):
+        name: str
+
+    class Color(StrEnum):
+        red = 'RED'
+
+    class Order(IntEnum):
+        one = 1
+
+    RootWithModel = RootModel[Model]
+    RootWithStr = RootModel[str]
+    RootWithInt = RootModel[int]
+    RootWithFloat = RootModel[float]
+    RootWithBool = RootModel[bool]
+    RootWithNone = RootModel[None]
+    # enums (which are subclasses of their base types)
+    RootWithColor = RootModel[Color]
+    RootWithOrder = RootModel[Order]
+
+    model = Model(name='with_model')
+    root_with_model = RootWithModel(root=model)
+    root_with_str = RootWithStr('with_str')
+    root_with_int = RootWithInt(-150)
+    root_with_float = RootWithFloat(2.0)
+    root_with_bool = RootWithBool(False)
+    root_with_none = RootWithNone(None)
+    root_with_color = RootWithColor(Color.red)
+    root_with_order = RootWithOrder(Order.one)
+
+    logfire.info(
+        'hi',
+        with_model=root_with_model,
+        with_str=root_with_str,
+        with_str_inner=root_with_str.root,
+        with_int=root_with_int,
+        with_int_inner=root_with_int.root,
+        with_float=root_with_float,
+        with_float_inner=root_with_float.root,
+        with_bool=root_with_bool,
+        with_bool_inner=root_with_bool.root,
+        with_none=root_with_none,
+        with_none_inner=root_with_none.root,
+        with_color=root_with_color,
+        with_color_inner=root_with_color.root,
+        with_order=root_with_order,
+        with_order_inner=root_with_order.root,
+    )
+
+    assert capsys.readouterr().out.splitlines() == snapshot(
+        [
+            'hi',
+            IsStr(),
+            '│ with_model=Model(',
+            "│                name='with_model',",
+            '│            )',
+            "│ with_str='with_str'",
+            "│ with_str_inner='with_str'",
+            '│ with_int=-150',
+            '│ with_int_inner=-150',
+            '│ with_float=2.0',
+            '│ with_float_inner=2.0',
+            '│ with_bool=False',
+            '│ with_bool_inner=False',
+            '│ with_none=None',
+            '│ with_none_inner=None',
+            "│ with_color=Color('RED')",
+            "│ with_color_inner=Color('RED')",
+            '│ with_order=Order(1)',
+            '│ with_order_inner=Order(1)',
         ]
     )
