@@ -11,7 +11,7 @@ from opentelemetry.sdk.trace.sampling import ALWAYS_OFF, ALWAYS_ON, Sampler, Sam
 import logfire
 from logfire._internal.constants import LEVEL_NUMBERS
 from logfire.sampling import SpanLevel, TailSamplingSpanInfo
-from logfire.testing import SeededRandomIdGenerator, TestExporter
+from logfire.testing import SeededRandomIdGenerator, TestExporter, TimeGenerator
 
 
 def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
@@ -39,7 +39,7 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
         [
             {
                 'name': 'notice',
-                'context': {'trace_id': 2, 'span_id': 4, 'is_remote': False},
+                'context': {'trace_id': 2, 'span_id': 3, 'is_remote': False},
                 'parent': None,
                 'start_time': 4000000000,
                 'end_time': 4000000000,
@@ -55,7 +55,7 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
             },
             {
                 'name': 'warn',
-                'context': {'trace_id': 4, 'span_id': 8, 'is_remote': False},
+                'context': {'trace_id': 4, 'span_id': 6, 'is_remote': False},
                 'parent': None,
                 'start_time': 8000000000,
                 'end_time': 8000000000,
@@ -70,9 +70,9 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
                 },
             },
             {
-                'name': 'span (pending)',
+                'name': 'span',
                 'context': {'trace_id': 5, 'span_id': 10, 'is_remote': False},
-                'parent': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'parent': {'trace_id': 5, 'span_id': 7, 'is_remote': False},
                 'start_time': 9000000000,
                 'end_time': 9000000000,
                 'attributes': {
@@ -86,9 +86,9 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
                 },
             },
             {
-                'name': 'span2 (pending)',
-                'context': {'trace_id': 5, 'span_id': 12, 'is_remote': False},
-                'parent': {'trace_id': 5, 'span_id': 11, 'is_remote': False},
+                'name': 'span2',
+                'context': {'trace_id': 5, 'span_id': 11, 'is_remote': False},
+                'parent': {'trace_id': 5, 'span_id': 8, 'is_remote': False},
                 'start_time': 10000000000,
                 'end_time': 10000000000,
                 'attributes': {
@@ -98,13 +98,13 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
                     'logfire.msg_template': 'span2',
                     'logfire.msg': 'span2',
                     'logfire.span_type': 'pending_span',
-                    'logfire.pending_parent_id': '0000000000000009',
+                    'logfire.pending_parent_id': '0000000000000007',
                 },
             },
             {
                 'name': 'error',
-                'context': {'trace_id': 5, 'span_id': 13, 'is_remote': False},
-                'parent': {'trace_id': 5, 'span_id': 11, 'is_remote': False},
+                'context': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'parent': {'trace_id': 5, 'span_id': 8, 'is_remote': False},
                 'start_time': 11000000000,
                 'end_time': 11000000000,
                 'attributes': {
@@ -119,8 +119,8 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
             },
             {
                 'name': 'span2',
-                'context': {'trace_id': 5, 'span_id': 11, 'is_remote': False},
-                'parent': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'context': {'trace_id': 5, 'span_id': 8, 'is_remote': False},
+                'parent': {'trace_id': 5, 'span_id': 7, 'is_remote': False},
                 'start_time': 10000000000,
                 'end_time': 12000000000,
                 'attributes': {
@@ -134,7 +134,7 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
             },
             {
                 'name': 'span',
-                'context': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'context': {'trace_id': 5, 'span_id': 7, 'is_remote': False},
                 'parent': None,
                 'start_time': 9000000000,
                 'end_time': 13000000000,
@@ -148,9 +148,9 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
                 },
             },
             {
-                'name': 'span3 (pending)',
-                'context': {'trace_id': 6, 'span_id': 15, 'is_remote': False},
-                'parent': {'trace_id': 6, 'span_id': 14, 'is_remote': False},
+                'name': 'span3',
+                'context': {'trace_id': 6, 'span_id': 13, 'is_remote': False},
+                'parent': {'trace_id': 6, 'span_id': 12, 'is_remote': False},
                 'start_time': 14000000000,
                 'end_time': 14000000000,
                 'attributes': {
@@ -166,8 +166,8 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
             },
             {
                 'name': 'trace',
-                'context': {'trace_id': 6, 'span_id': 16, 'is_remote': False},
-                'parent': {'trace_id': 6, 'span_id': 14, 'is_remote': False},
+                'context': {'trace_id': 6, 'span_id': 14, 'is_remote': False},
+                'parent': {'trace_id': 6, 'span_id': 12, 'is_remote': False},
                 'start_time': 15000000000,
                 'end_time': 15000000000,
                 'attributes': {
@@ -182,7 +182,7 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
             },
             {
                 'name': 'span3',
-                'context': {'trace_id': 6, 'span_id': 14, 'is_remote': False},
+                'context': {'trace_id': 6, 'span_id': 12, 'is_remote': False},
                 'parent': None,
                 'start_time': 14000000000,
                 'end_time': 16000000000,
@@ -200,7 +200,11 @@ def test_level_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
     )
 
 
-def test_duration_threshold(config_kwargs: dict[str, Any], exporter: TestExporter):
+def test_duration_threshold(
+    config_kwargs: dict[str, Any],
+    exporter: TestExporter,
+    time_generator: TimeGenerator,
+):
     # Set level to None to not include spans merely based on a high level.
     logfire.configure(
         **config_kwargs, sampling=logfire.SamplingOptions.level_or_duration(level_threshold=None, duration_threshold=3)
@@ -220,44 +224,20 @@ def test_duration_threshold(config_kwargs: dict[str, Any], exporter: TestExporte
         with logfire.span('span5'):
             logfire.info('long1')
 
+    # This reaches the duration threshold when span7 ends but before span6 ends.
+    # This means that a pending span is created for span6 but not for span7,
+    # because PendingSpanProcessor doesn't create pending spans for spans that have finished.
+    with logfire.span('span6'):
+        time_generator()
+        with logfire.span('span7'):
+            time_generator()
+
     assert exporter.exported_spans_as_dict(_include_pending_spans=True) == snapshot(
         [
-            {
-                'name': 'span4 (pending)',
-                'context': {'trace_id': 4, 'span_id': 10, 'is_remote': False},
-                'parent': {'trace_id': 4, 'span_id': 9, 'is_remote': False},
-                'start_time': 9000000000,
-                'end_time': 9000000000,
-                'attributes': {
-                    'code.filepath': 'test_tail_sampling.py',
-                    'code.function': 'test_duration_threshold',
-                    'code.lineno': 123,
-                    'logfire.msg_template': 'span4',
-                    'logfire.msg': 'span4',
-                    'logfire.span_type': 'pending_span',
-                    'logfire.pending_parent_id': '0000000000000000',
-                },
-            },
-            {
-                'name': 'span5 (pending)',
-                'context': {'trace_id': 4, 'span_id': 12, 'is_remote': False},
-                'parent': {'trace_id': 4, 'span_id': 11, 'is_remote': False},
-                'start_time': 10000000000,
-                'end_time': 10000000000,
-                'attributes': {
-                    'code.filepath': 'test_tail_sampling.py',
-                    'code.function': 'test_duration_threshold',
-                    'code.lineno': 123,
-                    'logfire.msg_template': 'span5',
-                    'logfire.msg': 'span5',
-                    'logfire.span_type': 'pending_span',
-                    'logfire.pending_parent_id': '0000000000000009',
-                },
-            },
             {
                 'name': 'long1',
-                'context': {'trace_id': 4, 'span_id': 13, 'is_remote': False},
-                'parent': {'trace_id': 4, 'span_id': 11, 'is_remote': False},
+                'context': {'trace_id': 4, 'span_id': 8, 'is_remote': False},
+                'parent': {'trace_id': 4, 'span_id': 7, 'is_remote': False},
                 'start_time': 11000000000,
                 'end_time': 11000000000,
                 'attributes': {
@@ -272,8 +252,8 @@ def test_duration_threshold(config_kwargs: dict[str, Any], exporter: TestExporte
             },
             {
                 'name': 'span5',
-                'context': {'trace_id': 4, 'span_id': 11, 'is_remote': False},
-                'parent': {'trace_id': 4, 'span_id': 9, 'is_remote': False},
+                'context': {'trace_id': 4, 'span_id': 7, 'is_remote': False},
+                'parent': {'trace_id': 4, 'span_id': 6, 'is_remote': False},
                 'start_time': 10000000000,
                 'end_time': 12000000000,
                 'attributes': {
@@ -287,7 +267,7 @@ def test_duration_threshold(config_kwargs: dict[str, Any], exporter: TestExporte
             },
             {
                 'name': 'span4',
-                'context': {'trace_id': 4, 'span_id': 9, 'is_remote': False},
+                'context': {'trace_id': 4, 'span_id': 6, 'is_remote': False},
                 'parent': None,
                 'start_time': 9000000000,
                 'end_time': 13000000000,
@@ -300,6 +280,52 @@ def test_duration_threshold(config_kwargs: dict[str, Any], exporter: TestExporte
                     'logfire.span_type': 'span',
                 },
             },
+            {
+                'name': 'span6',
+                'context': {'trace_id': 5, 'span_id': 11, 'is_remote': False},
+                'parent': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'start_time': 14000000000,
+                'end_time': 14000000000,
+                'attributes': {
+                    'code.filepath': 'test_tail_sampling.py',
+                    'code.function': 'test_duration_threshold',
+                    'code.lineno': 123,
+                    'logfire.msg_template': 'span6',
+                    'logfire.msg': 'span6',
+                    'logfire.span_type': 'pending_span',
+                    'logfire.pending_parent_id': '0000000000000000',
+                },
+            },
+            {
+                'name': 'span7',
+                'context': {'trace_id': 5, 'span_id': 10, 'is_remote': False},
+                'parent': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'start_time': 16000000000,
+                'end_time': 18000000000,
+                'attributes': {
+                    'code.filepath': 'test_tail_sampling.py',
+                    'code.function': 'test_duration_threshold',
+                    'code.lineno': 123,
+                    'logfire.msg_template': 'span7',
+                    'logfire.msg': 'span7',
+                    'logfire.span_type': 'span',
+                },
+            },
+            {
+                'name': 'span6',
+                'context': {'trace_id': 5, 'span_id': 9, 'is_remote': False},
+                'parent': None,
+                'start_time': 14000000000,
+                'end_time': 19000000000,
+                'attributes': {
+                    'code.filepath': 'test_tail_sampling.py',
+                    'code.function': 'test_duration_threshold',
+                    'code.lineno': 123,
+                    'logfire.msg_template': 'span6',
+                    'logfire.msg': 'span6',
+                    'logfire.span_type': 'span',
+                },
+            },
         ]
     )
 
@@ -319,7 +345,7 @@ def test_background_rate(config_kwargs: dict[str, Any], exporter: TestExporter):
     # None of them meet the tail sampling criteria.
     for _ in range(1000):
         logfire.info('info')
-    assert len(exporter.exported_spans) == 100 + 321
+    assert len(exporter.exported_spans) - 100 == snapshot(299)
 
 
 class TestSampler(Sampler):
@@ -380,7 +406,7 @@ def test_raw_head_sampler_with_tail_sampling(config_kwargs: dict[str, Any], expo
     # None of them meet the tail sampling criteria.
     for _ in range(1000):
         logfire.info('info')
-    assert len(exporter.exported_spans) == 100 + 315
+    assert len(exporter.exported_spans) - 100 == snapshot(293)
 
 
 def test_custom_head_and_tail(config_kwargs: dict[str, Any], exporter: TestExporter):
@@ -406,20 +432,20 @@ def test_custom_head_and_tail(config_kwargs: dict[str, Any], exporter: TestExpor
 
     for _ in range(1000):
         logfire.warn('warn')
-    assert span_counts == snapshot({'start': 720, 'end': 617})
-    assert len(exporter.exported_spans) == snapshot(103)
+    assert span_counts == snapshot({'start': 719, 'end': 611})
+    assert len(exporter.exported_spans) == snapshot(108)
     assert span_counts['end'] + len(exporter.exported_spans) == span_counts['start']
 
     exporter.clear()
     for _ in range(1000):
         with logfire.span('span'):
             pass
-    assert len(exporter.exported_spans_as_dict()) == snapshot(506)
+    assert len(exporter.exported_spans_as_dict()) == snapshot(511)
 
     exporter.clear()
     for _ in range(1000):
         logfire.error('error')
-    assert len(exporter.exported_spans) == snapshot(291)
+    assert len(exporter.exported_spans) == snapshot(298)
 
 
 def test_span_levels():
@@ -461,7 +487,7 @@ def test_span_levels():
 
 def test_invalid_rates():
     with inline_snapshot.extra.raises(
-        snapshot('ValueError: Invalid sampling rates, ' 'must be 0.0 <= background_rate <= head <= 1.0')
+        snapshot('ValueError: Invalid sampling rates, must be 0.0 <= background_rate <= head <= 1.0')
     ):
         logfire.SamplingOptions.level_or_duration(background_rate=-1)
     with pytest.raises(ValueError):
@@ -476,7 +502,7 @@ def test_trace_sample_rate(config_kwargs: dict[str, Any]):
     assert logfire.DEFAULT_LOGFIRE_INSTANCE.config.sampling.head == 0.123
     assert len(warnings) == 1
     assert str(warnings[0].message) == snapshot(
-        'The `trace_sample_rate` argument is deprecated. ' 'Use `sampling=logfire.SamplingOptions(head=...)` instead.'
+        'The `trace_sample_rate` argument is deprecated. Use `sampling=logfire.SamplingOptions(head=...)` instead.'
     )
 
 
